# Generated by Django 5.2.3 on 2025-11-14 08:44

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("marinerg_data_access", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="dataset",
            name="id_authority",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="dataset",
            name="keyword_vocabularies",
            field=models.CharField(blank=True, max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="dataset",
            name="keywords",
            field=models.CharField(blank=True, max_length=250),
        ),
        migrations.AddField(
            model_name="dataset",
            name="license",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="dataset",
            name="parent",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="children",
                to="marinerg_data_access.dataset",
            ),
        ),
    ]



from django.db import models

from ichec_django_core.models import Member, PopulatedForm, TimesStampMixin

from marinerg_facility.models import Facility, Equipment

from .dataset_template import DatasetTemplate


class DatasetTag(models.Model):

    value = models.CharField(max_length=200)


class Dataset(TimesStampMixin):

    creator = models.ForeignKey(Member, on_delete=models.CASCADE)

    title = models.CharField(max_length=250)

    description = models.TextField(blank=True)

    uri = models.CharField(max_length=250, null=True)

    id_authority = models.CharField(max_length=250, null=True)

    tags = models.ManyToManyField(DatasetTag, blank=True, related_name="datasets")

    license = models.CharField(max_length=250, null=True)

    parent = models.ForeignKey(
        "self", null=True, blank=True, related_name="children", on_delete=models.CASCADE
    )

    is_public = models.BooleanField(default=True)

    access_instructions = models.TextField(blank=True, null=True)

    equipment = models.ForeignKey(
        Equipment, null=True, related_name="datasets", on_delete=models.SET_NULL
    )

    facility = models.ForeignKey(
        Facility, null=True, related_name="datasets", on_delete=models.SET_NULL
    )

    template = models.ForeignKey(
        DatasetTemplate, null=True, related_name="datasets", on_delete=models.SET_NULL
    )

    form = models.OneToOneField(PopulatedForm, null=True, on_delete=models.CASCADE)



# Generated by Django 5.2.3 on 2025-11-07 08:28

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("ichec_django_core", "0001_initial"),
        (
            "marinerg_facility",
            "0004_equipment_created_at_equipment_updated_at_and_more",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="DatasetTemplate",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                ("title", models.CharField(max_length=250)),
                ("description", models.TextField(blank=True)),
                (
                    "creator",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.member",
                    ),
                ),
                (
                    "form",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.form",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Dataset",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                ("title", models.CharField(max_length=250)),
                ("description", models.TextField(blank=True)),
                ("uri", models.CharField(max_length=250, null=True)),
                ("is_public", models.BooleanField(default=True)),
                ("access_instructions", models.TextField(blank=True, null=True)),
                (
                    "creator",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.member",
                    ),
                ),
                (
                    "equipment",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="datasets",
                        to="marinerg_facility.equipment",
                    ),
                ),
                (
                    "facility",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="datasets",
                        to="marinerg_facility.facility",
                    ),
                ),
                (
                    "form",
                    models.OneToOneField(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.populatedform",
                    ),
                ),
                (
                    "template",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="datasets",
                        to="marinerg_data_access.datasettemplate",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from rest_framework import permissions

from ichec_django_core.models import Member
from ichec_django_core.view_sets import (
    SearchableModelViewSet,
    OwnerFullOrDjangoModelPermissions,
)

from marinerg_data_access.models import Dataset, DatasetTag
from marinerg_data_access.serializers import DatasetSerializer, DatasetTagSerializer


class DatasetPermissions(OwnerFullOrDjangoModelPermissions):
    owner_field = "creator"


class DatasetViewSet(SearchableModelViewSet):
    queryset = Dataset.objects.all()
    serializer_class = DatasetSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        DatasetPermissions,
    ]

    ordering_fields = SearchableModelViewSet.ordering_fields
    ordering: tuple[str, ...] = ("created_at",)
    search_fields = ["title", "facility__name", "equipment__name"]

    def perform_create(self, serializer):
        serializer.save(creator=Member.objects.get(id=self.request.user.id))


class DatasetTagViewSet(SearchableModelViewSet):
    queryset = DatasetTag.objects.all()
    serializer_class = DatasetTagSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]
    ordering_fields = ("value",)
    ordering: tuple[str, ...] = ("value",)
    search_fields = ["value"]

    def get_queryset(self):
        queryset = DatasetTag.objects.all()
        dataset_id = self.request.query_params.get("dataset")
        if dataset_id is not None:
            queryset = queryset.filter(dataset__id=dataset_id)
        return queryset



from rest_framework import permissions

from ichec_django_core.models import Member
from ichec_django_core.view_sets import (
    SearchableModelViewSet,
    OwnerFullOrDjangoModelPermissions,
)

from marinerg_data_access.models import DatasetTemplate
from marinerg_data_access.serializers import DatasetTemplateSerializer


class DatasetTemplatePermissions(OwnerFullOrDjangoModelPermissions):
    owner_field = "creator"


class DatasetTemplateViewSet(SearchableModelViewSet):
    queryset = DatasetTemplate.objects.all()
    serializer_class = DatasetTemplateSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        DatasetTemplatePermissions,
    ]

    ordering_fields = SearchableModelViewSet.ordering_fields
    ordering: tuple[str, ...] = ("created_at",)
    search_fields = ["title"]

    def perform_create(self, serializer):
        serializer.save(creator=Member.objects.get(id=self.request.user.id))



from pathlib import Path
import os
import argparse
import logging

import requests
import yaml

from ichec_django_core.utils.test_utils.portal_client import PortalClient, Paginated

from marinerg_facility.utils.test_utils.models import FacilityDetail

from marinerg_data_access.utils.test_utils.models import (
    DatasetDetail,
    DatasetTemplateCreate,
    DatasetCreate,
)

from .zenodo import ZenodoClient

logger = logging.getLogger(__name__)


class MarinergDataAccessClient(PortalClient):

    def __init__(
        self,
        username: str | None = None,
        password: str | None = None,
        endpoint: str = "http://localhost:8000/",
        zenodo_token: str | None = None,
    ):

        super().__init__(endpoint, username, password)
        self.zenodo_client = ZenodoClient(zenodo_token)
        self.doi_base = "https://doi.org/"

    def get_facilities(self, search_term="", page: int = 1) -> Paginated:
        return self.get_items(FacilityDetail, page)

    def get_datasets(self, query: dict | None = None, page: int = 1) -> Paginated:
        return self.get_items(DatasetDetail, page)

    def get_facility_datasets(
        self, facility: FacilityDetail, page: int = 1
    ) -> Paginated:
        return self.get_datasets({"facility": facility.id}, page)

    def get_data(self, dataset: DatasetDetail, output_dir: Path | None):
        if dataset.uri:
            self.get_data_by_doi(dataset.uri)

    def create_schema(self, schema: DatasetTemplateCreate):
        self.create_item(schema)

    def create_dataset(self, dataset: DatasetCreate):
        self.create_dataset(dataset)

    def get_data_by_doi(self, doi: str):

        r = requests.get(self.doi_base + doi)

        # Rely on doi.org redirect to get the real host and assign to a suitable
        # client for that host
        if "zenodo.org" in r.url:
            self.zenodo_client.get_data_by_url(r.url)
        else:
            raise RuntimeError(f"Host '{r.url}' not currently supported.")


def commom_init(args):

    zenodo_token = os.getenv("ZENODO_TOKEN")
    username = os.getenv("MARINERG_USERNAME", "consortium_admin")
    password = os.getenv("MARINERG_PASSWORD", "abc123")
    endpoint = os.getenv("MARINERG_ENDPOINT", "http://localhost:8000/")
    return username, password, endpoint, zenodo_token


def cli_schema_create(args):
    username, password, endpoint, _ = commom_init(args)

    client = MarinergDataAccessClient(username, password, endpoint)

    with open(args.path.resolve(), "r", encoding="utf-8") as f:
        schema_yaml = yaml.safe_load(f)
    schema = DatasetTemplateCreate(**schema_yaml)

    client.create_schema(schema)


def cli_dataset_create(args):
    username, password, endpoint, _ = commom_init(args)

    client = MarinergDataAccessClient(username, password, endpoint)

    with open(args.dataset.resolve(), "r", encoding="utf-8") as f:
        dataset_yaml = yaml.safe_load(f)

    if args.template:
        with open(args.dataset.resolve(), "r", encoding="utf-8") as f:
            template_yaml = yaml.safe_load(f)
        dataset_yaml = dataset_yaml | template_yaml

    dataset = DatasetCreate(**dataset_yaml)
    client.create_dataset(dataset)


def cli_data_download(args):
    username, password, endpoint, zenodo_token = commom_init(args)

    client = MarinergDataAccessClient(username, password, endpoint, zenodo_token)

    client.get_data_by_doi(args.doi)


def main_cli():
    parser = argparse.ArgumentParser()

    subparsers = parser.add_subparsers(required=True)

    schema_parser = subparsers.add_parser("schema")
    schema_subparsers = schema_parser.add_subparsers(required=True)

    schema_upload_parser = schema_subparsers.add_parser("create")

    schema_upload_parser.add_argument(
        "path",
        type=Path,
        help="Path to the schema file",
    )

    schema_upload_parser.set_defaults(func=cli_schema_create)

    data_parser = subparsers.add_parser("data")
    data_subparsers = data_parser.add_subparsers(required=True)

    data_download_parser = data_subparsers.add_parser("download")

    data_download_parser.add_argument(
        "dataset",
        type=int,
        help="Dataset id",
    )
    data_download_parser.set_defaults(func=cli_data_download)

    dataset_parser = subparsers.add_parser("dataset")
    dataset_subparsers = dataset_parser.add_subparsers(required=True)

    dataset_create_parser = dataset_subparsers.add_parser("create")

    dataset_create_parser.add_argument(
        "dataset",
        type=Path,
        help="Path to dataset",
    )

    dataset_create_parser.add_argument(
        "--template",
        type=Path,
        help="Path to dataset template",
    )
    dataset_create_parser.set_defaults(func=cli_dataset_create)

    args = parser.parse_args()
    args.func(args)



from ichec_django_core.serializers import (
    NestedHyperlinkedModelSerializer,
    FormSerializer,
)

from ..models import DatasetTemplate


class DatasetTemplateSerializer(NestedHyperlinkedModelSerializer):

    form = FormSerializer()

    class Meta:
        model = DatasetTemplate
        fields = NestedHyperlinkedModelSerializer.base_fields + (
            "creator",
            "title",
            "description",
            "form",
        )
        read_only_fields = NestedHyperlinkedModelSerializer.base_fields + ("creator",)

    def create(self, validated_data):

        form = validated_data.pop("form")
        form_instance = FormSerializer().create(form)

        many_to_many = self.pop_many_to_many(validated_data)
        instance = DatasetTemplate.objects.create(form=form_instance, **validated_data)
        self.add_many_to_many(instance, many_to_many)
        return instance

    def update(self, instance, validated_data):
        form = validated_data.pop("form")

        instance = super().update(instance, validated_data)

        serializer = FormSerializer()
        serializer.update(instance.form, form)

        return instance



from .dataset import DatasetSerializer, DatasetTagSerializer
from .dataset_template import DatasetTemplateSerializer

__all__ = ["DatasetSerializer", "DatasetTagSerializer", "DatasetTemplateSerializer"]



from rest_framework import serializers

from ichec_django_core.serializers import (
    NestedHyperlinkedModelSerializer,
    PopulatedFormSerializer,
)

from ..models import Dataset, DatasetTag


class DatasetTagSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = DatasetTag
        fields = ("value", "id", "url")
        read_only_fields = ("value", "id", "url")


class DatasetSerializer(NestedHyperlinkedModelSerializer):

    form = PopulatedFormSerializer()

    class Meta:
        model = Dataset
        fields = NestedHyperlinkedModelSerializer.base_fields + (
            "creator",
            "title",
            "description",
            "uri",
            "id_authority",
            "license",
            "parent",
            "is_public",
            "access_instructions",
            "equipment",
            "facility",
            "template",
            "form",
            "tags",
        )
        read_only_fields = NestedHyperlinkedModelSerializer.base_fields + ("creator",)

    def create(self, validated_data):

        form = validated_data.pop("form")
        if form:
            form_instance = PopulatedFormSerializer().create(form)
        else:
            form = None

        many_to_many = self.pop_many_to_many(validated_data)
        instance = Dataset.objects.create(form=form_instance, **validated_data)
        self.add_many_to_many(instance, many_to_many)
        return instance

    def update(self, instance, validated_data):
        form = validated_data.pop("form")

        instance = super().update(instance, validated_data)

        serializer = PopulatedFormSerializer()
        serializer.update(instance.form, form)

        return instance



# Generated by Django 5.2.3 on 2025-11-14 08:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "marinerg_data_access",
            "0002_dataset_id_authority_dataset_keyword_vocabularies_and_more",
        ),
    ]

    operations = [
        migrations.RemoveField(
            model_name="dataset",
            name="keyword_vocabularies",
        ),
        migrations.RemoveField(
            model_name="dataset",
            name="keywords",
        ),
    ]



from ichec_django_core.utils.test_utils.models import (
    FormCreate,
    FormGroupCreate,
    FormFieldCreate,
    PopulatedFormCreate,
    FormFieldValueCreate,
)

from .models import (
    DatasetCreate,
    DatasetTemplateDetail,
    DatasetTemplateCreate,
)


def create_dataset_templates(
    count: int = 10,
    offset: int = 0,
) -> list[DatasetTemplateCreate]:

    contents = []
    for idx in range(offset, count + offset):
        fields = [
            FormFieldCreate(
                label="Measurement Unit",
                key="measurement_unit",
                field_type="TEXT",
                required=True,
                default="",
                description="Enter the base measurement unit used.",
            ),
            FormFieldCreate(
                label="Notes",
                key="notes",
                field_type="TEXT",
                required=False,
                default="",
                description="Add supporting test notes.",
            ),
        ]

        groups = [FormGroupCreate(fields=fields)]

        contents.append(
            DatasetTemplateCreate(
                title=f"Dataset Template {idx}",
                description=f"Description of dataset template {idx}",
                form=FormCreate(groups=groups),
            )
        )
    return contents


def create_datasets(template: DatasetTemplateDetail, count: int = 10, offset: int = 0):

    unit_field = template.get_field("measurement_unit")
    if not unit_field:
        raise RuntimeError("Field not found")

    contents = []
    for idx in range(offset, count + offset):
        contents.append(
            DatasetCreate(
                title=f"Scripted dataset {idx}",
                description=f"Description of scripted dataset {idx}",
                uri=f"http://www.dataset_{idx}.org",
                form=PopulatedFormCreate(
                    values=[
                        FormFieldValueCreate(
                            field=unit_field.id, value="measurement_unit"
                        )
                    ]
                ),
            )
        )
    return contents



from django.db import models

from ichec_django_core.models import Member, Form, TimesStampMixin


class DatasetTemplate(TimesStampMixin):

    creator = models.ForeignKey(Member, on_delete=models.CASCADE)

    title = models.CharField(max_length=250)

    description = models.TextField(blank=True)

    form = models.OneToOneField(Form, on_delete=models.CASCADE)



import requests


class ZenodoClient:

    def __init__(self, token: str | None = None):
        self.token = token
        self.api_root = "https://zenodo.org/api/"

    def get_data_by_url(self, url: str):

        record_id = url.split("/")[-1]
        # search = f"{self.zenodo_root}records/
        # ?q=doi:{doi}&size=10&page=1&all_versions=1"

        if self.token:
            headers = {"Authorization": f"Bearer {self.token}"}
        else:
            headers = {}
        r = requests.get(f"{self.api_root}records/{record_id}", headers=headers)

        record = r.json()
        for file_record in record["files"]:
            key = file_record["key"]
            link = file_record["links"]["self"]

            with requests.get(link, stream=True) as r:
                r.raise_for_status()
                with open(key, "wb") as f:
                    for chunk in r.iter_content(chunk_size=8192):
                        f.write(chunk)
            break



# Generated by Django 5.2.3 on 2025-11-14 09:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("marinerg_data_access", "0003_remove_dataset_keyword_vocabularies_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="DatasetTag",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("value", models.CharField(max_length=200)),
            ],
        ),
        migrations.AddField(
            model_name="dataset",
            name="tags",
            field=models.ManyToManyField(
                blank=True,
                related_name="datasets",
                to="marinerg_data_access.datasettag",
            ),
        ),
    ]



from pydantic import BaseModel
from ichec_django_core.utils.test_utils.models import (
    Timestamped,
    FormCreate,
    FormDetail,
    FormFieldDetail,
    register_types,
    PopulatedFormCreate,
    PopulatedFormDetail,
)


class DatasetBase(BaseModel, frozen=True):

    title: str
    description: str = ""
    uri: str | None = None
    is_public: bool = True
    access_instructions: str | None = None
    equipment: str | None = None
    facility: str | None = None
    template: str | None = None


class DatasetCreate(DatasetBase, frozen=True):

    form: PopulatedFormCreate | None = None


class DatasetDetail(Timestamped, DatasetBase, frozen=True):

    form: PopulatedFormDetail | None = None
    creator: str


class DatasetList(DatasetDetail, frozen=True):
    pass


class DatasetTemplateBase(BaseModel, frozen=True):

    title: str
    description: str = ""
    uri: str | None = None
    is_public: bool = True
    access_instructions: str | None = None
    equipment: str | None = None
    facility: str | None = None
    template: str | None = None


class DatasetTemplateCreate(DatasetTemplateBase, frozen=True):

    form: FormCreate


class DatasetTemplateDetail(Timestamped, DatasetTemplateBase, frozen=True):

    form: FormDetail
    creator: str

    def get_field(self, key: str) -> FormFieldDetail | None:
        for group in self.form.groups:
            for field in group.fields:
                if field.key == key:
                    return field
        return None


class DatasetTemplateList(DatasetTemplateDetail, frozen=True):
    pass


register_types(
    "datasets",
    {"create": DatasetCreate, "detail": DatasetDetail, "list": DatasetList},
)

register_types(
    "dataset_templates",
    {"create": DatasetTemplateCreate, "detail": DatasetTemplateDetail},
)



from .view_sets import DatasetViewSet, DatasetTagViewSet, DatasetTemplateViewSet


def register_drf_views(router):
    router.register(r"datasets", DatasetViewSet)
    router.register(r"dataset_tags", DatasetTagViewSet)
    router.register(r"dataset_templates", DatasetTemplateViewSet)


urlpatterns: list = []
